part of '../flutterx_live_data.dart';

extension LiveDataExt<X> on LiveData<X> {
  LiveData<Y> map<Y>(Y Function(X) transform, {bool reactive = true}) => MediatorLiveData<Y>()
    ..addTransformSource<X>(this, transform)
    ..reactive = reactive;

  void observe(StateObserver context, Observer<X> onChanged) => context._observe<X>(this, onChanged);
}

extension MediatorExt<Y> on MediatorMixin<Y> {
  void addTransformSource2<X1, X2>(LiveData<X1> x1, LiveData<X2> x2, Y Function(X1, X2) transform) =>
      addTransformSources([x1, x2], (values) => transform(values[0], values[1]));

  void addTransformSource3<X1, X2, X3>(
          LiveData<X1> x1, LiveData<X2> x2, LiveData<X3> x3, Y Function(X1, X2, X3) transform) =>
      addTransformSources([x1, x2, x3], (values) => transform(values[0], values[1], values[2]));

  void addTransformSource4<X1, X2, X3, X4>(
          LiveData<X1> x1, LiveData<X2> x2, LiveData<X3> x3, LiveData<X4> x4, Y Function(X1, X2, X3, X4) transform) =>
      addTransformSources([x1, x2, x3, x4], (values) => transform(values[0], values[1], values[2], values[4]));
}

extension StateObserverExt on StateObserver {
  void observe2<X1, X2>(LiveData<X1> x1, LiveData<X2> x2, void Function(X1, X2) onChanged) =>
      observeAll([x1, x2], (values) => onChanged(values[0], values[1]));

  void observe3<X1, X2, X3>(LiveData<X1> x1, LiveData<X2> x2, LiveData<X3> x3, void Function(X1, X2, X3) onChanged) =>
      observeAll([x1, x2, x3], (values) => onChanged(values[0], values[1], values[2]));

  void observe4<X1, X2, X3, X4>(LiveData<X1> x1, LiveData<X2> x2, LiveData<X3> x3, LiveData<X4> x4,
          void Function(X1, X2, X3, X4) onChanged) =>
      observeAll([x1, x2, x3, x4], (values) => onChanged(values[0], values[1], values[2], values[3]));
}
