part of '../flutterx_live_data.dart';

class MutableLiveData<T> extends LiveData<T> {
  MutableLiveData.late();

  MutableLiveData({T? initialValue}) : assert(initialValue is T, 'initialValue must be of type $T, got $initialValue') {
    setData(initialValue as T);
  }

  set value(T value) => setData(value);
}

class MutableMediatorLiveData<T> extends MutableLiveData<T> with MediatorMixin<T> {
  MutableMediatorLiveData.late() : super.late();

  MutableMediatorLiveData({T? initialValue}) : super(initialValue: initialValue);
}

class HistoryMutableLiveData<T> extends MutableLiveData<T> with HistoryMixin<T> {
  HistoryMutableLiveData.late() : super.late();

  HistoryMutableLiveData({T? initialValue}) : super(initialValue: initialValue);
}

class StorageMutableLiveData<T> extends MutableLiveData<T> with StorageMixin<T> {
  @override
  final String key;

  @override
  final Map<String, String> _storage;

  @override
  final T _defaultValue;

  @override
  final String Function(T value) _encode;

  StorageMutableLiveData({
    required this.key,
    required Map<String, String> storage,
    T? defaultValue,
    String Function(T value) encode = _defaultEncode,
    T Function(String source)? decode,
  })  : assert(defaultValue is T, 'defaultValue must be of type $T, got $defaultValue'),
        _storage = storage,
        _encode = encode,
        _defaultValue = defaultValue as T,
        super(initialValue: StorageMixin.resolve(key, storage, defaultValue, decode ?? _defaultDecode));

  static String _defaultEncode(dynamic value) => value.toString();

  static T _defaultDecode<T>(String source) {
    assert(source is T,
        'Default decode only supports ${source.runtimeType}, got $T. Use a custom decode function instead');
    return source as T;
  }
}
