part of '../flutterx_live_data.dart';

mixin StateObserver<T extends StatefulWidget> on State<T> {
  final Map<int, _DisposableObserver> _disposables = {};
  int _key = 0;

  @override
  void initState() {
    super.initState();
    _doRegister();
  }

  @override
  void didUpdateWidget(T oldWidget) {
    super.didUpdateWidget(oldWidget);
    _doRegister();
  }

  @override
  void dispose() {
    super.dispose();
    for (final disposable in _disposables.values) disposable.dispose();
  }

  void registerObservers(BuildContext context);

  void observeAll(Iterable<LiveData> dataList, void Function(List values) onChanged) {
    dataList = dataList.toList(growable: false);
    for (final data in dataList)
      _observe(data, (_) => onChanged(dataList.map((data) => data._data).toList(growable: false)));
  }

  void _doRegister() => WidgetsBinding.instance!.addPostFrameCallback((_) {
        _key = 0;
        registerObservers(context);
        assert(_disposables.length == _key);
      });

  void _observe<V>(LiveData<V> data, Observer<V> onChanged) {
    final key = _key++;
    final existing = _disposables[key];
    if (existing != null) {
      if (existing.updated(data, onChanged)) return;
      existing.dispose();
    }
    _disposables[key] = _DisposableObserver<V>(data..observeForever(onChanged), onChanged);
  }
}

@immutable
class _DisposableObserver<T> {
  final LiveData<T> _data;
  final Observer<T> _onChanged;

  const _DisposableObserver(this._data, this._onChanged);

  bool updated(LiveData data, Object onChanged) => identical(_data, data) && identical(_onChanged, onChanged);

  void dispose() => _data.removeObserver(_onChanged);
}
