part of '../flutterx_live_data.dart';

typedef LiveWidgetBuilder<T> = Widget Function(BuildContext context, T value);

class LiveDataBuilder<T> extends StatelessWidget {
  final LiveData<T> data;

  final LiveWidgetBuilder<T> builder;

  final Widget? placeholder;

  const LiveDataBuilder({Key? key, required this.data, required this.builder, this.placeholder}) : super(key: key);

  @override
  StatelessElement createElement() => _LiveDataBuilderElement<T>(this);

  @override
  Widget build(BuildContext context) => _liveBuild<T>(context, data, builder, placeholder);
}

class _LiveDataBuilderElement<T> extends StatelessElement with _LiveDataBuilderMixin<T, LiveDataBuilder<T>> {
  _LiveDataBuilderElement(LiveDataBuilder<T> widget) : super(widget);

  @override
  bool dataChanged(LiveDataBuilder<T> oldWidget, LiveDataBuilder<T> newWidget) =>
      !identical(oldWidget.data, newWidget.data);

  @override
  void link(LiveDataBuilder<T> widget) => widget.data.observeForever(onChanged, dispatcher: Dispatcher.postFrame);

  @override
  void unlink(LiveDataBuilder<T> widget) => widget.data.removeObserver(onChanged);
}

Widget _liveBuild<T>(BuildContext context, LiveData<T> data, LiveWidgetBuilder<T> builder, Widget? placeholder) =>
    data.initialized && data.isActive ? builder(context, data._data) : (placeholder ?? const SizedBox.shrink());

mixin _LiveDataBuilderMixin<T, W extends StatelessWidget> on StatelessElement {
  @override
  W get widget => super.widget as W;

  @override
  void mount(Element? parent, dynamic newSlot) {
    super.mount(parent, newSlot);
    link(widget);
  }

  @override
  void update(W newWidget) {
    final oldWidget = widget;
    super.update(newWidget);
    if (dataChanged(oldWidget, newWidget)) {
      unlink(oldWidget);
      link(newWidget);
    }
  }

  @override
  void unmount() {
    unlink(widget);
    super.unmount();
  }

  bool dataChanged(W oldWidget, W newWidget);

  void link(W widget);

  void unlink(W widget);

  void onChanged(T _) => markNeedsBuild();
}
