part of '../flutterx_live_data.dart';

class CombinedLiveDataBuilder<T> extends StatelessWidget {
  final List<LiveData> data;

  final T Function(List values) transform;

  final LiveWidgetBuilder<T> builder;

  final Widget? placeholder;

  const CombinedLiveDataBuilder(
      {Key? key, required this.data, required this.transform, required this.builder, this.placeholder})
      : super(key: key);

  static CombinedLiveDataBuilder<Y> with2<X1, X2, Y>(
          {Key? key,
          required LiveData<X1> x1,
          required LiveData<X2> x2,
          required Y Function(X1, X2) transform,
          required LiveWidgetBuilder<Y> builder,
          Widget? placeholder}) =>
      CombinedLiveDataBuilder<Y>(
          data: [x1, x2], transform: (values) => transform(values[0], values[1]), builder: builder);

  static CombinedLiveDataBuilder<Y> with3<X1, X2, X3, Y>(
          {Key? key,
          required LiveData<X1> x1,
          required LiveData<X2> x2,
          required LiveData<X3> x3,
          required Y Function(X1, X2, X3) transform,
          required LiveWidgetBuilder<Y> builder,
          Widget? placeholder}) =>
      CombinedLiveDataBuilder<Y>(
          data: [x1, x2, x3], transform: (values) => transform(values[0], values[1], values[2]), builder: builder);

  static CombinedLiveDataBuilder<Y> with4<X1, X2, X3, X4, Y>(
          {Key? key,
          required LiveData<X1> x1,
          required LiveData<X2> x2,
          required LiveData<X3> x3,
          required LiveData<X4> x4,
          required Y Function(X1, X2, X3, X4) transform,
          required LiveWidgetBuilder<Y> builder,
          Widget? placeholder}) =>
      CombinedLiveDataBuilder<Y>(
          data: [x1, x2, x3, x4],
          transform: (values) => transform(values[0], values[1], values[2], values[3]),
          builder: builder);

  @override
  StatelessElement createElement() => _CombinedLiveDataBuilderElement<T>(this);

  @override
  Widget build(BuildContext context) =>
      _liveBuild<T>(context, (context as _CombinedLiveDataBuilderElement<T>).mediator, builder, placeholder);
}

class _CombinedLiveDataBuilderElement<T> extends StatelessElement
    with _LiveDataBuilderMixin<T, CombinedLiveDataBuilder<T>> {
  final MediatorLiveData<T> mediator = MediatorLiveData<T>();

  _CombinedLiveDataBuilderElement(CombinedLiveDataBuilder<T> widget) : super(widget);

  @override
  bool dataChanged(CombinedLiveDataBuilder<T> oldWidget, CombinedLiveDataBuilder<T> newWidget) =>
      !listEquals(oldWidget.data, newWidget.data);

  @override
  void link(CombinedLiveDataBuilder<T> widget) {
    mediator.addTransformSources(widget.data, widget.transform);
    mediator.observeForever(onChanged, dispatcher: Dispatcher.postFrame);
  }

  @override
  void unlink(CombinedLiveDataBuilder<T> widget) {
    mediator.removeObserver(onChanged);
    mediator.removeSources(widget.data);
  }
}
